package org.opencalaccess.invites.eo;


import java.util.UUID;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.webobjects.eocontrol.EOEditingContext;
import com.webobjects.foundation.NSArray;

import er.extensions.eof.ERXQ;
import er.extensions.qualifiers.ERXAndQualifier;

/**
 * A UserInvite stores the data needed to let a user into an application. An
 * invite can be either permanent or expiring. If it is expiring, it can either
 * be valid or expired. A ticket is a kind of invite which says the user can
 * receive expiring invites at will.
 *
 * The attributes of an invite are:
 * <ul>
 * <li>A link to the user, to get access to the user's email address.
 * (required)</li>
 * <li>A link to the right, which specifies the app and the access allowed to
 * the app. (required)</li>
 * <li>A digest to be sent to the user. (optional)</li>
 * <li>The date/time (is system milliseconds) that the invite was created.
 * (optional)</li>
 * <li>The date/time (in system milliseconds) that the invite will expire.
 * (optional)</li>
 * </ul>
 *
 * A permanent invite has a digest, a create date, and no expire date.
 *
 * An expiring invite has a digest, a create date, and an expire date.
 *
 * A ticket does not have a digest, it does have a create date, and the expire
 * date is actually a survival interval for invites generated for this ticket.
 *
 */
public class UserInvite extends _UserInvite {

	private static final long serialVersionUID = 1042479077058548834L;

	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(UserInvite.class);

	private static final long TICKET_INTERVAL_MAX = 1000000000000L;

	public static UserInvite createUserInvite(EOEditingContext ec, Long created, User user, UserRight userRight) {
		UserInvite nextInvite = _UserInvite.createUserInvite(ec, created, user, userRight);
		nextInvite.setDigest(UUID.randomUUID().toString());
		return nextInvite;
	}

	public static NSArray<UserInvite> allPermanentInvites(EOEditingContext ec) {

		ERXAndQualifier qualifier = ERXQ.isNull(UserInvite.EXPIRES_KEY).and(ERXQ.isNotNull(UserInvite.CREATED_KEY))
				.and(ERXQ.isNotNull(UserInvite.DIGEST_KEY));

		return UserInvite.fetchUserInvites(ec, qualifier, null);
	}

	public static NSArray<UserInvite> allExpiringInvites(EOEditingContext ec) {

		ERXAndQualifier qualifier = ERXQ.isNotNull(UserInvite.EXPIRES_KEY).and(ERXQ.isNotNull(UserInvite.CREATED_KEY))
				.and(ERXQ.isNotNull(UserInvite.DIGEST_KEY));

		return UserInvite.fetchUserInvites(ec, qualifier, null);
	}

	public static NSArray<UserInvite> allTickets(EOEditingContext ec) {

		ERXAndQualifier qualifier = ERXQ.isNotNull(UserInvite.CREATED_KEY).and(ERXQ.isNull(UserInvite.DIGEST_KEY))
				.and(ERXQ.lessThan(UserInvite.EXPIRES_KEY, TICKET_INTERVAL_MAX));

		return UserInvite.fetchUserInvites(ec, qualifier, null);
	}

	public static UserInvite inviteForDigest(EOEditingContext ec, String digest) {
		return UserInvite.fetchUserInvite(ec, ERXQ.is(UserInvite.DIGEST_KEY, digest));
	}

	public static UserInvite permanentInviteForUserAndApp(EOEditingContext ec, String email, String appName) {

		// * A permanent invite has a digest, a create date, and no expire date.

		ERXAndQualifier qualifier = ERXQ.isNotNull(UserInvite.DIGEST_KEY).and(ERXQ.isNotNull(UserInvite.CREATED_KEY))
				.and(ERXQ.isNull(UserInvite.EXPIRES_KEY))
				.and(ERXQ.is(UserInvite.USER_KEY + "." + User.EMAIL_ADDRESS_KEY, email))
				.and(ERXQ.is(UserInvite.USER_RIGHT_KEY + "." + UserRight.APP_NAME_KEY, appName));

		NSArray<UserInvite> found = UserInvite.fetchUserInvites(ec, qualifier, null);

		return (found.size() > 0) ? found.get(0) : null;
	}

	public static UserInvite unexpiredInviteForUserAndApp(EOEditingContext ec, String email, String appName) {
		return inviteForUserAndApp(ec, email, appName, false);
	}

	public static UserInvite expectedInviteForUserAndApp(EOEditingContext ec, String email, String appName) {
		return inviteForUserAndApp(ec, email, appName, true);
	}

	private static UserInvite inviteForUserAndApp(EOEditingContext ec, String email, String appName, boolean expected) {

		// * An expiring invite has a digest, a create date, and an expire date.

		ERXAndQualifier qualifier = ERXQ.isNotNull(UserInvite.DIGEST_KEY).and(ERXQ.isNotNull(UserInvite.CREATED_KEY))
				.and(ERXQ.lessThan(UserInvite.EXPIRES_KEY, System.currentTimeMillis()))
				.and(ERXQ.is(UserInvite.USER_KEY + "." + User.EMAIL_ADDRESS_KEY, email))
				.and(ERXQ.is(UserInvite.USER_RIGHT_KEY + "." + UserRight.APP_NAME_KEY, appName));

		NSArray<UserInvite> found = UserInvite.fetchUserInvites(ec, qualifier, null);

		if (expected && found == null) {
			System.err.println("Could not find invite: " + email + ", " + appName);
		}

		return (found.size() > 0) ? found.get(0) : null;
	}

	public static UserInvite ticketForUserAndApp(EOEditingContext ec, String email, String appName) {

		// * A ticket does not have a digest, it does have a create date, and the
		// * expire date is actually a survival interval for invites generated for
		// * this ticket.

		ERXAndQualifier qualifier = ERXQ.isNull(UserInvite.DIGEST_KEY).and(ERXQ.isNotNull(UserInvite.CREATED_KEY))
				.and(ERXQ.isNotNull(UserInvite.EXPIRES_KEY))
				.and(ERXQ.is(UserInvite.USER_KEY + "." + User.EMAIL_ADDRESS_KEY, email))
				.and(ERXQ.is(UserInvite.USER_RIGHT_KEY + "." + UserRight.APP_NAME_KEY, appName));

		NSArray<UserInvite> found = UserInvite.fetchUserInvites(ec, qualifier, null);

		return (found.size() > 0) ? found.get(0) : null;
	}
}
