package org.opencalaccess.leginfo;

import java.io.File;

import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.opencalaccess.U;
import org.opencalaccess.leginfo.eo.Bill;
import org.opencalaccess.leginfo.eo.BillVersion;

import com.webobjects.foundation.NSArray;
import com.wounit.rules.MockEditingContext;

public class BillVersionTest {

	@Rule
    public MockEditingContext ec = new MockEditingContext("LegData");

	private String ab_1000_bill_xml = U.stringInFile(new File(System.getProperty("user.dir") + "/Resources/tests/ab_1000_bill_text.xml"));

	private String ab_110_bill_xml = U.stringInFile(new File(System.getProperty("user.dir") + "/Resources/tests/ab_110_bill_text.xml"));

	private BillVersion exampleVersion() {
		BillVersion version = null;
		Bill bill = Bill.createBill(
				ec,
				"202120220AB1",
				"20210AB100099INT",
				new Long(1000),
				"Amended Assembly",
				"AB",
				"0",
				"20212022",
				version);
		version = BillVersion.createBillVersion(ec, "202120220AB1", "2021-02-18", "20210AB100099INT", 99, bill);
		return version;
	}

	@Test
	public void testBillVersionDigestTextShort() {

		BillVersion version = exampleVersion();

		version.setBillXml(ab_1000_bill_xml);

		String expected = "<p>Existing law, the Planning and Zoning Law, requires each city, county, and city " +
				"and county to prepare and adopt a general plan that contains certain mandatory elements, including a " +
				"housing element. Existing law defines several terms for the purposes of these provisions.</p>" +
				"<p>This bill would make nonsubstantive changes to those provisions.</p>";

		Assert.assertEquals(expected, version.digestText());
	}

	@Test
	public void testBillVersionDigestTextParts() {

		BillVersion version = exampleVersion();

		version.setBillXml(ab_110_bill_xml);

		NSArray<String> parts = version.digestTextParts();

		Assert.assertTrue(parts.get(0).startsWith("Existing law provides for the payment of unemployment compensation"));

		Assert.assertTrue(parts.get(4).startsWith("The California Constitution requires"));

		Assert.assertTrue(parts.get(6).startsWith("This bill would declare that it is to take effect immediately"));

		Assert.assertEquals(7, parts.size());
	}

	@Test
	public void testBillVersionDigestTextWithAddsAndDeletes() {

		BillVersion version = exampleVersion();

		version.setBillXml(ab_110_bill_xml);

		String digestPart = version.digestTextParts().get(2);

		String expected = "This bill would require the Department of Corrections and Rehabilitation to provide " +
				"the names and social security numbers of current inmates to the Employment Development Department " +
				"for the purposes of preventing payments on fraudulent claims for unemployment compensation " +
				"benefits.<s> The bill would also require a county to provide the names and social security " +
				"numbers of inmates currently serving a sentence in the county’s jail to the Employment " +
				"Development Department for those same purposes.</s> The bill would require the names and social " +
				"security numbers to be provided to the Employment Development Department on the first of every " +
				"month and upon the Employment Development Department’s request. Because this bill would expand " +
				"the scope of an existing<s> crime and would impose additional duties on local " +
				"officials,</s><b> crime,</b> the bill imposes a state-mandated local program.";

		Assert.assertEquals(expected, digestPart);
	}

	/**
	 * Not testing any code, but rather, confirming how these regexes work.
	 */
	@Test
	public void testThisBillWouldsRegexLogic() {

		String str1 = "Existing law defines several terms for the purposes of these provisions.</p>";

		String str2 = "<p>This bill would make nonsubstantive changes to those provisions.</p>";

		Assert.assertTrue(str2.matches(".*[Tt]his.*bill.*would.*"));

		Assert.assertFalse(str1.matches(".*[Tt]his.*bill.*would.*"));
	}

	@Test
	public void testThisBillWouldsSimple() {

		BillVersion version = exampleVersion();

		version.setBillXml(ab_1000_bill_xml);

		Assert.assertEquals(
				"This bill would make nonsubstantive changes to those provisions.",
				version.thisBillWoulds().get(0));

		Assert.assertEquals(1, version.thisBillWoulds().size());
	}

	@Test
	public void testThisBillWouldsMore() {

		BillVersion version = exampleVersion();

		version.setBillXml(ab_110_bill_xml);

		Assert.assertEquals(version.digestTextParts().get(2), version.thisBillWoulds().get(0));
		Assert.assertEquals(version.digestTextParts().get(3), version.thisBillWoulds().get(1));
		Assert.assertEquals(version.digestTextParts().get(5), version.thisBillWoulds().get(2));
		Assert.assertEquals(version.digestTextParts().get(6), version.thisBillWoulds().get(3));

		Assert.assertEquals(4, version.thisBillWoulds().size());
	}
}
