package org.opencalaccess.leginfo.eo;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import er.extensions.eof.ERXKey;
import er.extensions.eof.ERXKey.Type;

/**
 * An Authoring captures the same content as the bill_version_authors_tbl table, but with
 * a more sensible twist.
 *
 * <pre>bill_version <----->> bill_version_authors_tbl</pre>
 *
 * A row in the bill_version_authors_tbl points to a single person. Since there are multiple
 * authors for a bill, this relationship is one bill_version goes to many bill_version_authors_tbl
 * rows.
 *
 * But when an author is a committee, the bill_version_authors_tbl row contains references to many people
 * itself.
 *
 * The author table flattens this. The contribution values in the bill_version_authors_tbl are:
 *
 * <ul>
 * <li>LEAD_AUTHOR</li>
 * <li>PRINCIPAL_COAUTHOR</li>
 * <li>COAUTHOR</li>
 * </ul>
 *
 * The authorings table adds new contribution values:
 *
 * <ul>
 * <li>COMMITTEE_CHAIR</li>
 * <li>COMMITTEE_VICE_CHAIR</li>
 * <li>COMMITTEE_MEMBER</li>
 * </ul>
 *
 * This enables us to flatten the original table so that each row in the authorings table points
 * to only one entity. If a committee is a lead author, then that is still true, but the members of
 * the committee are added in with their proper contribution value.
 *
 * This does have within it the assumption that only one committee is an author. That seems a
 * reasonable assumption.
 *
 * If the author in the authoring row is a person, then there will be a link to the legislator_tbl.
 * This cannot exist for a committee.
 *
 * TODO I currently have a pointer to the bill_version_id. I should use the pk of the
 * bill_version_authors_tbl table and, when I need the bill, get it through this object.
 *
 * @author ray 
 *
 */
public class Authoring extends _Authoring {

	private static final long serialVersionUID = 1643228732757884896L;

	@SuppressWarnings("unused")
	private static final Logger log = LoggerFactory.getLogger(Authoring.class);

	public static final String LEAD_AUTHOR = "LEAD_AUTHOR";
	public static final String PRINCIPAL_COAUTHOR =   "PRINCIPAL_COAUTHOR";
	public static final String COAUTHOR = "COAUTHOR";
	public static final String COMMITTEE_CHAIR = "COMMITTEE_CHAIR";
	public static final String COMMITTEE_VICE_CHAIR = "COMMITTEE_VICE_CHAIR";
	public static final String COMMITTEE_MEMBER = "COMMITTEE_MEMBER";

	// Attribute Keys
	public static final ERXKey<BillVersion> BILL_VERSION = new ERXKey<BillVersion>("billVersion", Type.Attribute);

	// Attributes
	public static final String BILL_VERSION_KEY = BILL_VERSION.key();

	public BillVersion billVersion() {
		return this.billVersionAuthor().billVersion();
	}

	public Legislator legislator() {
		return this.author().legislator();
	}
}
